--- chan_sip.c.old	2005-03-02 18:38:27.000000000 +0100
+++ chan_sip.c	2005-09-21 16:36:29.000000000 +0200
@@ -1707,6 +1707,15 @@
 		return 0;
 	}
 	ast_mutex_lock(&p->lock);
+	/* Go ahead and free RTP port. Bugfix #3986 */
+	if (p->rtp) {
+		ast_rtp_destroy(p->rtp);
+		p->rtp = NULL;
+	}
+	if (p->vrtp) {
+		ast_rtp_destroy(p->vrtp);
+		p->vrtp = NULL;
+	}	
 #ifdef OSP_SUPPORT
 	if ((p->osphandle > -1) && (ast->_state == AST_STATE_UP)) {
 		ast_osp_terminate(p->osphandle, AST_CAUSE_NORMAL, p->ospstart, time(NULL) - p->ospstart);
@@ -2196,6 +2205,9 @@
 	/* Retrieve audio/etc from channel.  Assumes p->lock is already held. */
 	struct ast_frame *f;
 	static struct ast_frame null_frame = { AST_FRAME_NULL, };
+	if (!p->rtp)
+		return &null_frame;
+	
 	switch(ast->fdno) {
 	case 0:
 		f = ast_rtp_read(p->rtp);	/* RTP Audio */
@@ -2660,7 +2672,8 @@
 		if ((sscanf(m, "audio %d RTP/AVP %n", &x, &len) == 1)) {
 			portno = x;
 			/* Scan through the RTP payload types specified in a "m=" line: */
-			ast_rtp_pt_clear(p->rtp);
+			if (p->rtp)
+				ast_rtp_pt_clear(p->rtp);
 			codecs = m + len;
 			while(!ast_strlen_zero(codecs)) {
 				if (sscanf(codecs, "%d%n", &codec, &len) != 1) {
@@ -2669,7 +2682,8 @@
 				}
 				if (debug)
 					ast_verbose("Found RTP audio format %d\n", codec);
-				ast_rtp_set_m_type(p->rtp, codec);
+				if (p->rtp)
+					ast_rtp_set_m_type(p->rtp, codec);
 				codecs += len;
 				/* Skip over any whitespace */
 				while(*codecs && (*codecs < 33)) codecs++;
@@ -2738,13 +2752,15 @@
 	  if (debug)
 		ast_verbose("Found description format %s\n", mimeSubtype);
 	  /* Note: should really look at the 'freq' and '#chans' params too */
-	  ast_rtp_set_rtpmap_type(p->rtp, codec, "audio", mimeSubtype);
+	  if (p->rtp)
+			ast_rtp_set_rtpmap_type(p->rtp, codec, "audio", mimeSubtype);
 	  if (p->vrtp)
 		  ast_rtp_set_rtpmap_type(p->vrtp, codec, "video", mimeSubtype);
 	}
 
 	/* Now gather all of the codecs that were asked for: */
-	ast_rtp_get_current_formats(p->rtp,
+	if (p->rtp)	
+		ast_rtp_get_current_formats(p->rtp,
 				&peercapability, &peernoncodeccapability);
 	if (p->vrtp)
 		ast_rtp_get_current_formats(p->vrtp,
@@ -2791,7 +2807,8 @@
 			} else {
 				ast_moh_start(p->owner->bridge, NULL);
 				if (sendonly)
-					ast_rtp_stop(p->rtp);
+					if (p->rtp)
+						ast_rtp_stop(p->rtp);
 			}
 		}
 	}
@@ -3525,7 +3542,8 @@
 		return -1;
 	}
 	respprep(&resp, p, msg, req);
-	ast_rtp_offered_from_local(p->rtp, 0);
+	if (p->rtp)
+		ast_rtp_offered_from_local(p->rtp, 0);
 	add_sdp(&resp, p);
 	return send_response(p, &resp, retrans, seqno);
 }
@@ -3605,7 +3623,8 @@
 		reqprep(&req, p, "INVITE", 0, 1);
 	
 	add_header(&req, "Allow", ALLOWED_METHODS);
-	ast_rtp_offered_from_local(p->rtp, 1);
+	if (p->rtp)
+		ast_rtp_offered_from_local(p->rtp, 1);
 	add_sdp(&req, p);
 	/* Use this as the basis */
 	copy_request(&p->initreq, &req);
@@ -3777,7 +3796,8 @@
 	}
 	add_header(&req, "Allow", ALLOWED_METHODS);
 	if (sdp) {
-		ast_rtp_offered_from_local(p->rtp, 1);
+		if (p->rtp)
+			ast_rtp_offered_from_local(p->rtp, 1);
 		add_sdp(&req, p);
 	} else {
 		add_header(&req, "Content-Length", "0");
